
love.graphics.setDefaultFilter("nearest", "nearest")
love.graphics.setLineStyle("rough")

local window_width, window_height = love.graphics.getDimensions()
local game_width = 320
local game_height = 180
local scale_x = window_width/game_width
local scale_y = window_height/game_height
local canvas = love.graphics.newCanvas()
local image = love.graphics.newImage("sprites.png")
local iw, ih = image:getDimensions()
local sprites = {
    ["Air"] = false,
    ["Wall"] = love.graphics.newQuad(0, 0, 16, 16, iw, ih),
    ["Slope"] = love.graphics.newQuad(16, 0, 16, 16, iw, ih),
}
local tiles = {}
local tilewidth = 16
local map_width = 20
local map_height = 11
local tile_ids = {
    [0] = {name="Air", rotation=0},
    [1] = {name="Wall", rotation=0},
    [2] = {name="Slope", rotation=0}, --up left
    [3] = {name="Slope", rotation=90}, -- up right
    [4] = {name="Slope", rotation=-90}, -- down left
    [5] = {name="Slope", rotation=180}, -- down right
}
local tilemap = {
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
    {1, 4, 1, 1, 1, 1, 1, 1, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {1, 0, 4, 1, 1, 1, 1, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {1, 0, 0, 4, 1, 1, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1, 3, 0, 0, 0, 1, 1, 1, 1},
    {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 1, 1, 0, 0, 0, 1, 1, 1, 1},
    {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 5, 0, 0, 0, 1, 1, 1, 1},
    {1, 0, 0, 2, 1, 1, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {1, 0, 2, 1, 1, 1, 1, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
    {1, 2, 1, 1, 1, 1, 1, 1, 3, 0, 1, 1, 3, 0, 0, 0, 1, 1, 1, 1},
    {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
}


local function createMapTiles()
    for y = 1, map_height do
        for x = 1, map_width do
            local tile = {}
            tile.x = x*tilewidth-tilewidth
            tile.y = y*tilewidth-tilewidth
            tile.type = tile_ids[tilemap[y][x]]
            tile.sprite = sprites[tile.type.name]
            tile.rotation = tile.type.rotation
            tile.width = tilewidth
            tiles[#tiles+1] = tile
        end
    end
end

local function updateCanvas()
    love.graphics.setCanvas(canvas)
    love.graphics.clear()
    for key, tile in ipairs(tiles) do
        if tile.sprite then
            local x, y = tile.x+tile.width/2, tile.y+tile.width/2
            local r = math.rad(tile.rotation)
            local sx, sy = 1, 1
            local kx, ky = tile.width/2, tile.width/2
            love.graphics.draw(image, tile.sprite, x, y, r, sx, sy, kx, ky)
            if tile.type.name == "Slope" then
                love.graphics.rectangle("line", tile.x, tile.y, tile.width, tile.width)
            end
        end
    end
    love.graphics.setCanvas()
end

-- Main callbacks --
function love.load()
    createMapTiles()
    updateCanvas()
end

function love.update(dt)
end

function love.draw()
    love.graphics.push()
    love.graphics.scale(scale_x, scale_y)
    love.graphics.draw(canvas, 0, 2)
    love.graphics.pop()
    love.graphics.print("Map Prototyping v1 - 06-10-2022")
end

function love.keypressed(key)
    if key == "escape" then love.event.quit() end
end
